@@ -51,46 +51,47 @@
 #' @param cores the number of cores used for parallel computing. Default = 1.
 #' @param valid.proportion the proportion of target data to be used as validation data when detecting transferable sources. Useful only when \code{transfer.source.id = "auto"}. Default = \code{NULL}, meaning that the cross-validation will be applied.
 #' @param valid.nfolds the number of folds used in cross-validation procedure when detecting transferable sources. Useful only when \code{transfer.source.id = "auto"} and \code{valid.proportion = NULL}. Default = 3.
-#' @param lambda a vector indicating the choice of lambdas in transferring, debiasing and detection steps. Should be a vector with names "transfer", "debias", and "detection", each component of which can be either "lambda.min" or "lambda.1se". Component \code{transfer} is the lambda (the penalty parameter) used in transferrring step. Component \code{debias} is the lambda used in debiasing step. Component \code{detection} is the lambda used in the transferable source detection algorithm. Default choice of \code{lambda.transfer} and \code{lambda.detection} are "lambda.1se", while default \code{lambda.debias} = "lambda.min". If the user wants to change the default setting, input a vector with corresponding \code{lambda.transfer}/\code{lambda.debias}/\code{lambda.detection} names and corresponding values. Examples: lambda = list(transfer = "lambda.min", debias = "lambda.1se"); lambda = list(transfer = "lambda.min", detection = "lambda.min").
+#' @param lambda a vector indicating the choice of lambdas in transferring, debiasing and detection steps. Should be a vector with names "transfer", "debias", and "detection", each component of which can be either "lambda.min" or "lambda.1se". Component \code{transfer} is the lambda (the penalty parameter) used in transferrring step. Component \code{debias} is the lambda used in debiasing step. Component \code{detection} is the lambda used in the transferable source detection algorithm. Default choice of \code{lambda.transfer} and \code{lambda.detection} are "lambda.1se", while default \code{lambda.debias} = "lambda.min". If the user wants to change the default setting, input a vector with corresponding \code{lambda.transfer}/\code{lambda.debias}/\code{lambda.detection} names and corresponding values. Examples: lambda = list(transfer = "lambda.1se", debias = "lambda.min", detection = "lambda.1se"); lambda = list(transfer = 1, debias = 0.5, detection = 1).
 #' \itemize{
 #' \item "lambda.min": value of lambda that gives minimum mean cross-validated error in the sequence of lambda.
 #' \item "lambda.1se": largest value of lambda such that error is within 1 standard error of the minimum.
 #' }
+#' @param lambda.seq the sequence of lambda candidates used in the algorithm. Should be a list of three vectors with names "transfer", "debias", and "detection". Default = list(transfer = NULL, debias = NULL, detection = NULL). "NULL" means the algorithm will determine the sequence automatically, based on the same method used in \code{cv.glmnet}.
 #' @param detection.info the logistic flag indicating whether to print detection information or not. Useful only when \code{transfer.source.id = "auto"}. Default = \code{TURE}.
 #' @param target.weights weight vector for each target instance. Should be a vector with the same length of target response. Default = \code{NULL}, which makes all instances equal-weighted.
 #' @param source.weights a list of weight vectors for the instances from each source. Should be a list with the same length of the number of sources. Default = \code{NULL}, which makes all instances equal-weighted.
-#' @param C0 the constant used in the transferable source detection algorithm. See Algorithm 2 in Tian, Y. and Feng, Y., 2021. Default = 2.
+#' @param C0 the constant used in the transferable source detection algorithm. See Algorithm 2 in Tian, Y. & Feng, Y. (2023). Default = 2.
 #' @param ... additional arguments.
 #' @return An object with S3 class \code{"glmtrans"}.
 #' \item{beta}{the estimated coefficient vector.}
 #' \item{family}{the response type.}
-#' \item{transfer.source.id}{the transferable souce index. If in the input, \code{transfer.source.id = 1:length(source)} or \code{transfer.source.id = "all"}, then the outputed \code{transfer.source.id = 1:length(source)}. If the inputed \code{transfer.source.id = "auto"}, only transferable source detected by the algorithm will be outputed.}
+#' \item{transfer.source.id}{the transferable source index. If in the input, \code{transfer.source.id = 1:length(source)} or \code{transfer.source.id = "all"}, then the outputed \code{transfer.source.id = 1:length(source)}. If the inputed \code{transfer.source.id = "auto"}, only transferable source detected by the algorithm will be outputed.}
 #' \item{fitting.list}{a list of other parameters of the fitted model.}
 #' \itemize{
-#' \item{w_a}{the estimator obtained from the transferring step.}
-#' \item{delta_a}{the estimator obtained from the debiasing step.}
-#' \item{target.valid.loss}{the validation (or cross-validation) loss on target data. Only available when \code{transfer.source.id = "auto"}.}
-#' \item{source.loss}{the loss on each source data. Only available when \code{transfer.source.id = "auto"}.}
-#' \item{threshold}{the threshold to determine transferability. Only available when \code{transfer.source.id = "auto"}.}
+#' \item \code{w_a}: the estimator obtained from the transferring step.
+#' \item \code{delta_a}: the estimator obtained from the debiasing step.
+#' \item \code{target.valid.loss}: the validation (or cross-validation) loss on target data. Only available when \code{transfer.source.id = "auto"}.
+#' \item \code{source.loss}: the loss on each source data. Only available when \code{transfer.source.id = "auto"}.
+#' \item \code{threshold}: the threshold to determine transferability. Only available when \code{transfer.source.id = "auto"}.
 #' }
 #' @seealso \code{\link{predict.glmtrans}}, \code{\link{source_detection}}, \code{\link{models}}, \code{\link{plot.glmtrans}}, \code{\link[glmnet]{cv.glmnet}}, \code{\link[glmnet]{glmnet}}.
 #' @references
-#' Tian, Y. and Feng, Y., 2021. \emph{Transfer Learning under High-dimensional Generalized Linear Models. arXiv preprint arXiv:2105.14328.}
+#' Tian, Y., & Feng, Y. (2023). \emph{Transfer learning under high-dimensional generalized linear models. Journal of the American Statistical Association, 118(544), 2684-2697.}
 #'
-#' Li, S., Cai, T.T. and Li, H., 2020. \emph{Transfer learning for high-dimensional linear regression: Prediction, estimation, and minimax optimality. arXiv preprint arXiv:2006.10593.}
+#' Li, S., Cai, T.T. & Li, H. (2020). \emph{Transfer learning for high-dimensional linear regression: Prediction, estimation, and minimax optimality. arXiv preprint arXiv:2006.10593.}
 #'
-#' Friedman, J., Hastie, T. and Tibshirani, R., 2010. \emph{Regularization paths for generalized linear models via coordinate descent. Journal of statistical software, 33(1), p.1.}
+#' Friedman, J., Hastie, T. & Tibshirani, R. (2010). \emph{Regularization paths for generalized linear models via coordinate descent. Journal of statistical software, 33(1), p.1.}
 #'
-#' Zou, H. and Hastie, T., 2005. \emph{Regularization and variable selection via the elastic net. Journal of the royal statistical society: series B (statistical methodology), 67(2), pp.301-320.}
+#' Zou, H. & Hastie, T. (2005). \emph{Regularization and variable selection via the elastic net. Journal of the royal statistical society: series B (statistical methodology), 67(2), pp.301-320.}
 #'
-#' Tibshirani, R., 1996. \emph{Regression shrinkage and selection via the lasso. Journal of the Royal Statistical Society: Series B (Methodological), 58(1), pp.267-288.}
+#' Tibshirani, R. (1996). \emph{Regression shrinkage and selection via the lasso. Journal of the Royal Statistical Society: Series B (Methodological), 58(1), pp.267-288.}
 #'
 #' @examples
 #' set.seed(0, kind = "L'Ecuyer-CMRG")
 #'
 #' # fit a linear regression model
 #' D.training <- models("gaussian", type = "all", n.target = 100, K = 2, p = 500)
-#' D.test <- models("gaussian", type = "target", n.target = 100, p = 500)
+#' D.test <- models("gaussian", type = "target", n.target = 500, p = 500)
 #' fit.gaussian <- glmtrans(D.training$target, D.training$source)
 #' y.pred.glmtrans <- predict(fit.gaussian, D.test$target$x)
 #'
@@ -105,7 +106,7 @@
 #' \donttest{
 #' # fit a logistic regression model
 #' D.training <- models("binomial", type = "all", n.target = 100, K = 2, p = 500)
-#' D.test <- models("binomial", type = "target", n.target = 100, p = 500)
+#' D.test <- models("binomial", type = "target", n.target = 500, p = 500)
 #' fit.binomial <- glmtrans(D.training$target, D.training$source, family = "binomial")
 #' y.pred.glmtrans <- predict(fit.binomial, D.test$target$x, type = "class")
 #'
@@ -120,7 +121,7 @@
 #'
 #' # fit a Poisson regression model
 #' D.training <- models("poisson", type = "all", n.target = 100, K = 2, p = 500)
-#' D.test <- models("poisson", type = "target", n.target = 100, p = 500)
+#' D.test <- models("poisson", type = "target", n.target = 500, p = 500)
 #' fit.poisson <- glmtrans(D.training$target, D.training$source, family = "poisson")
 #' y.pred.glmtrans <- predict(fit.poisson, D.test$target$x, type = "response")
 #'
@@ -136,6 +137,7 @@ glmtrans <- function(target, source = NULL, family = c("gaussian", "binomial", "
                      transfer.source.id = "auto", alpha = 1, standardize = TRUE, intercept = TRUE,
                      nfolds = 10, cores = 1, valid.proportion = NULL, valid.nfolds = 3,
                      lambda = c(transfer = "lambda.1se", debias = "lambda.min", detection = "lambda.1se"),
+                     lambda.seq = list(transfer = NULL, debias = NULL, detection = NULL),
                      detection.info = TRUE, target.weights = NULL, source.weights = NULL, C0 = 2, ...) {
 
   family <- match.arg(family)
@@ -169,7 +171,7 @@ glmtrans <- function(target, source = NULL, family = c("gaussian", "binomial", "
     transfer.source.id <- 1:length(source)
   } else if (!is.null(source) && is.string(transfer.source.id) && transfer.source.id == "auto") { # automatically check which source data set to transfer
     A <- source_detection(target = target, family = family, source = source, alpha = alpha,
-                          cores = cores,  lambda = lambda["detection"], valid.proportion = valid.proportion,
+                          cores = cores,  lambda = lambda["detection"], lambda.seq = lambda.seq$detection, valid.proportion = valid.proportion,
                           valid.nfolds = valid.nfolds, detection.info = detection.info, standardize = standardize,
                           intercept = intercept, nfolds = nfolds, target.weights = target.weights, source.weights = source.weights, C0 = C0, ...)
 
@@ -213,9 +215,10 @@ glmtrans <- function(target, source = NULL, family = c("gaussian", "binomial", "
   }, simplify = FALSE))
   n.try <- 0
   while (T) {
-    cv.fit.trans <- try(cv.glmnet(x = all.x, y = all.y, family = family, alpha = alpha, weights = w, nfolds = nfolds, parallel = I(cores > 1),
-                                  intercept = intercept, standardize = standardize, lambda.min.ratio = 0.01,...), silent = T)
-    if (class(cv.fit.trans) != "try-error") {
+    cv.fit.trans <- try(cv.glmnet(x = all.x, y = all.y, family = family, alpha = alpha, weights = w, nfolds = nfolds,
+                                  parallel = I(cores > 1), intercept = intercept, standardize = standardize, lambda.min.ratio = 0.01,
+                                  lambda = lambda.seq$transfer, ...), silent = T)
+    if (!inherits(cv.fit.trans, "try-error")) {
       break
     }
     n.try <- n.try + 1
@@ -243,8 +246,10 @@ glmtrans <- function(target, source = NULL, family = c("gaussian", "binomial", "
 
   n.try <- 0
   while (T) {
-    cv.fit.correct <- try(cv.glmnet(x = as.matrix(target$x), y = target$y, weights = target.weights, offset = offset, family = family, parallel = I(cores > 1), intercept = intercept, lambda.min.ratio = 0.01, nfolds = nfolds, ...), silent=TRUE)
-    if (class(cv.fit.correct) != "try-error") {
+    cv.fit.correct <- try(cv.glmnet(x = as.matrix(target$x), y = target$y, weights = target.weights, offset = offset, family = family,
+                                    parallel = I(cores > 1), intercept = intercept, lambda.min.ratio = 0.01, nfolds = nfolds,
+                                    lambda = lambda.seq$debias, ...), silent=TRUE)
+    if (!inherits(cv.fit.correct, "try-error")) {
       break
     }
     n.try <- n.try + 1
